import type { PeerId } from "@libp2p/interface-peer-id";
import type { ConnectionManager } from "@libp2p/interface-connection-manager";
import type { Registrar } from "@libp2p/interface-registrar";
import type { PubSub } from "@libp2p/interface-pubsub";
import type { Startable } from "@libp2p/interfaces/startable";
import { EventEmitter } from "@libp2p/interfaces/events";
import type { Node } from "@canvas-js/okra";
import type { Signature } from "@canvas-js/signed-cid";
import type { IPLDValue, Message } from "@canvas-js/interfaces";
import { MessageSigner } from "#store";
import { SyncOptions } from "./SyncService.js";
import { Awaitable } from "./utils.js";
export type GossipLogComponents = {
    peerId: PeerId;
    registrar: Registrar;
    connectionManager: ConnectionManager;
    pubsub?: PubSub;
};
export interface GossipLogInit {
    location?: string | null;
    sync?: boolean;
}
export type GossipLogEvents = {
    sync: CustomEvent<{
        topic: string;
        peerId: PeerId;
    }>;
    commit: CustomEvent<{
        topic: string;
        root: Node;
    }>;
    message: CustomEvent<{
        topic: string;
        id: string;
        signature: Signature | null;
        message: Message;
        result: void | IPLDValue;
    }>;
};
export interface TopicInit<Payload, Result> extends SyncOptions {
    apply: (id: string, signature: Signature | null, message: Message<Payload>) => Awaitable<Result>;
    validate: (payload: unknown) => payload is Payload;
    signatures?: boolean;
    sequencing?: boolean;
    replay?: boolean;
}
export declare class GossipLog extends EventEmitter<GossipLogEvents> implements Startable {
    #private;
    private readonly components;
    readonly location: string | null;
    private readonly sync;
    private readonly log;
    constructor(components: GossipLogComponents, init: GossipLogInit);
    isStarted(): boolean;
    start(): Promise<void>;
    afterStart(): Promise<void>;
    beforeStop(): Promise<void>;
    stop(): Promise<void>;
    subscribe<Payload, Result>(topic: string, init: TopicInit<Payload, Result>): Promise<void>;
    unsubscribe(topic: string): Promise<void>;
    publish<Payload, Result>(topic: string, payload: Payload, options?: {
        signer?: MessageSigner<Payload>;
    }): Promise<{
        id: string;
        result: Awaited<Result>;
        recipients: Promise<never[] | PeerId[]>;
    }>;
    private handleMessage;
}
export declare const gossiplog: (init: GossipLogInit) => (components: GossipLogComponents) => GossipLog;
